#ifndef ARRAY2_H
#define ARRAY2_H

#include <cassert>
#include <algorithm>
#include <vector>

template<class T>
struct Array2
{
   int nx, ny;
   std::vector<T> a;

   Array2(void)
      : nx(0), ny(0)
   {}

   Array2(int nx_, int ny_)
      : nx(nx_), ny(ny_), a(nx_*ny_)
   {}

   Array2(int nx_, int ny_, const T &value)
      : nx(nx_), ny(ny_), a(nx_*ny_, value)
   {}

   void assign(int nx_, int ny_, const T& value)
   {
      resize(nx_, ny_);
      assign(value);
   }

   void assign(const T& value)
   {
      for(unsigned int i=0; i<a.size(); ++i)
         a[i]=value;
   }

   void clear(void)
   {
      nx=0; ny=0;
      a.clear();
   }

   bool empty(void)
   { return nx==0 && ny==0; }

   void reserve(int nx_, int ny_)
   { a.reserve(nx_*ny_); }

   void resize(int nx_, int ny_)
   {
      nx=nx_; ny=ny_;
      a.resize(nx*ny);
   }

   void swap(Array2 &b)
   {
      std::swap(nx, b.nx);
      std::swap(ny, b.ny);
      a.swap(b.a);
   }

   T* data(void)
   { return &a[0]; }

   T &operator()(int i, int j)
   {
      assert(i<nx && j<ny);
      return a[i+nx*j];
   }

   T &operator[](const int i)
   {
      assert(i >= 0 && i<nx*ny);
      return a[i];
   }

   const T &operator()(int i, int j) const
   {
      assert(i<nx && j<ny);
      return a[i+nx*j];
   }
   
   const T &operator[](int i) const
   {
      assert(i >= 0 && i<nx*ny);
      return a[i];
   }
   
   size_t size() const
   {
      return a.size();
   }
   
};

typedef Array2<double>         Array2d;
typedef Array2<float>          Array2f;
typedef Array2<long>           Array2l;
typedef Array2<unsigned long>  Array2ul;
typedef Array2<int>            Array2i;
typedef Array2<unsigned int>   Array2ui;
typedef Array2<short>          Array2s;
typedef Array2<unsigned short> Array2us;
typedef Array2<char>           Array2c;
typedef Array2<unsigned char>  Array2uc;
//typedef Array2<bool>           Array2b;  NOTE: this doesn't work as intended since std::vector<bool> is specialized

template<class T>
bool operator==(const Array2<T>& a, const Array2<T>& b)
{ return a.nx==b.nx && a.ny==b.ny && a.a==b.a; }

template<class T>
bool operator!=(const Array2<T>& a, const Array2<T>& b)
{ return !(a==b); }

template<class T>
bool operator<(const Array2<T>& a, const Array2<T>& b)
{
   if (a.nx<b.nx) return true;
   else if(a.nx==b.nx){
      if(a.ny<b.ny) return true;
      else if(a.ny==b.ny)
         return a.a<b.a;
   }
   return false;
}

template<class T>
bool operator>(const Array2<T>& a, const Array2<T>& b)
{ return b<a; }

template<class T>
bool operator<=(const Array2<T>& a, const Array2<T>& b)
{ return !(b<a); }

template<class T>
bool operator>=(const Array2<T>& a, const Array2<T>& b)
{ return !(a<b); }

#endif
