#ifndef BARNES_HUT_H
#define BARNES_HUT_H

#include "vec.h"

struct BarnesHutTree
{
   Vec2d xmin, xmax; // bounds on the extent of this tree
   double total_mass; // for all points included in this tree
   Vec2d centre_of_mass; // for all points included in this tree
   double bounding_radius; // all points in this tree shoudl be within this distance of centre_of_mass
   bool leaf; // indicate if this node is a leaf (no children)
   std::vector<int> point_indices; // only for leaf nodes
   BarnesHutTree *children[2]; // only for non-leaf nodes

   BarnesHutTree(void);
   ~BarnesHutTree(void);
   void clear(void);
   void construct_from_points(int n, const Vec2d *x, const double *mass, int max_depth=30, int max_points_per_leaf=1);
   void evaluate_gravity(int n, const Vec2d *x, const double *mass, Vec2d *accel, const double error_tolerance,
                         const double gravitational_constant=6.67428e-11);

   protected:

   void construct_recursively(int n, const Vec2d *x, const double *mass,
                              int max_depth, int max_points_per_leaf, int current_depth, int num_indices, int *indices);

   // add the contribution of this node (of the tree) to point i's acceleration
   // You will need to implement this!
   void add_gravity(int n, const Vec2d *x, const double *mass, Vec2d *accel, int i, const double error_tolerance);
};

#endif

