#include <cstdarg>
#include <fstream>
#include "writesgi.h"

// used here and there for filenames etc.
#define BUFFER_LENGTH 1024
#ifdef _MSC_VER
#define vsnprintf _vsnprintf
#endif

static void write_big_endian_ushort(std::ostream &output, unsigned short v)
{
   output.put((v>>8)%256);
   output.put(v%256);
}

static void write_big_endian_uint(std::ostream &output, unsigned int v)
{
   output.put((v>>24)%256);
   output.put((v>>16)%256);
   output.put((v>>8)%256);
   output.put(v%256);
}

void write_sgi(const Array2<Vec3f> &image, const char *filename_format, ...)
{
   va_list ap;
   va_start(ap, filename_format);
   char filename[BUFFER_LENGTH];
   vsnprintf(filename, BUFFER_LENGTH, filename_format, ap);
   std::ofstream output(filename, std::ofstream::binary);
   if(!output) return;
   // first write the SGI header
   write_big_endian_ushort(output, 474); // magic number to identify this as an SGI image file
   output.put(0); // uncompressed
   output.put(1); // use 8-bit colour depth
   write_big_endian_ushort(output, 3); // number of dimensions
   write_big_endian_ushort(output, image.nx); // x size
   write_big_endian_ushort(output, image.ny); // y size
   write_big_endian_ushort(output, 3); // three colour channels (z size)
   write_big_endian_uint(output, 0); // minimum pixel value
   write_big_endian_uint(output, 255); // maximum pixel value
   write_big_endian_uint(output, 0); // dummy spacing
   // image name
   int i;
   for(i=0; i<80 && filename[i]; ++i)
      output.put(filename[i]);
   for(; i<80; ++i)
      output.put(0);
   write_big_endian_uint(output, 0); // colormap is normal
   for(i=0; i<404; ++i) output.put(0); // filler to complete header
   // now write the SGI image data
   unsigned char *scanline=new unsigned char[image.nx];
   for(int channel=0; channel<3; ++channel) for(int j=0; j<image.ny; ++j){
      for(int i=0; i<image.nx; ++i) scanline[i]=(unsigned char)(clamp(image(i,j)[channel],0.f,1.f)*255);
      output.write((const char*)scanline, image.nx);
   }
   delete[] scanline;
}

